package com.link.core.cache;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.parser.Feature;
import com.link.base.user.model.CoreUser;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.thread.LinkThreadPoolExecutor;
import com.link.core.util.BeanTools;
import com.link.core.util.PasswordUtil;
import com.link.core.util.RedisUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import com.link.core.util.redisclient.LinkRedisClient;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.script.Bindings;
import javax.script.Compilable;
import javax.script.CompiledScript;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.script.SimpleBindings;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CacheHandler {
   Logger logger = LoggerFactory.getLogger(CacheHandler.class);
   private static ScriptEngine javaScriptEngine;
   private static final String PREFIX = "lnk_dc_";
   private static final String CACHE_KEY = "CACHE_KEY";

   private Object getData(CacheAopProxyChain pjp) throws Throwable {
      try {
         return this.getData(pjp, pjp.getArgs());
      } catch (InvocationTargetException var3) {
         InvocationTargetException e = var3;
         throw new Exception(e.getCause());
      }
   }

   public Object getData(CacheAopProxyChain pjp, Object[] arguments) throws Throwable {
      long startTime = System.currentTimeMillis();
      Object result = pjp.doProxyChain(arguments);
      long useTime = System.currentTimeMillis() - startTime;
      String className = pjp.getTarget().getClass().getName();
      this.logger.info("{}.{}, use time:{}ms", new Object[]{className, pjp.getMethod().getName(), useTime});
      return result;
   }

   private Object getDataByHasRead(CacheAopProxyChain pjp, LinkCache cache) throws Throwable {
      LinkRedisClient linkRedisClient = null;

      Object var8;
      try {
         linkRedisClient = RedisUtil.getJedis();
         Map<String, String> result = this.getCacheKey(pjp, cache);
         String key = (String)result.get("key");
         String reloadCache = (String)result.get("reloadCache");
         Object obj;
         if (linkRedisClient.exists(key) && "N".equals(reloadCache)) {
            obj = this.getRealType(linkRedisClient.get(key), pjp.getMethod());
            return obj;
         }

         obj = this.getData(pjp);
         if (linkRedisClient.hexists("CACHE_KEY", cache.expire())) {
            linkRedisClient.set(key, JSON.toJSONString(obj));
            linkRedisClient.expire(key, Integer.valueOf(linkRedisClient.hget("CACHE_KEY", cache.expire())));
         }

         var8 = obj;
      } catch (Exception var12) {
         Exception e = var12;
         throw new Exception(e);
      } finally {
         RedisUtil.returnResource(linkRedisClient);
      }

      return var8;
   }

   private Object getRealType(String string, Method method) {
      Type returnType = method.getGenericReturnType();
      return JSON.parseObject(string, returnType, new Feature[0]);
   }

   private Object getDataByLast(CacheAopProxyChain pjp, LinkCache cache) throws Throwable {
      LinkRedisClient linkRedisClient = null;

      Object obj;
      try {
         linkRedisClient = RedisUtil.getJedis();
         Map<String, String> result = this.getCacheKey(pjp, cache);
         String key = (String)result.get("key");
         String reloadCache = (String)result.get("reloadCache");
         if (!linkRedisClient.exists(key) || !"N".equals(reloadCache)) {
            obj = this.getData(pjp);
            if (linkRedisClient.hexists("CACHE_KEY", cache.expire())) {
               linkRedisClient.set(key, JSON.toJSONString(obj));
               linkRedisClient.expire(key, Integer.valueOf(linkRedisClient.hget("CACHE_KEY", cache.expire())));
            }

            Object var8 = obj;
            return var8;
         }

         this.asycSaveObjToCache(key, pjp, (Object)null, cache);
         obj = this.getRealType(linkRedisClient.get(key), pjp.getMethod());
      } catch (Exception var12) {
         Exception e = var12;
         throw new Exception(e);
      } finally {
         RedisUtil.returnResource(linkRedisClient);
      }

      return obj;
   }

   private Object getDataByNeeds(CacheAopProxyChain pjp, LinkCache cache) throws Throwable {
      LinkRedisClient linkRedisClient = null;

      Object var7;
      try {
         linkRedisClient = RedisUtil.getJedis();
         Object obj = this.getData(pjp, pjp.getArgs());
         this.asycSaveObjToCache((String)null, pjp, obj, cache);
         Object var13 = obj;
         return var13;
      } catch (Exception var11) {
         Map<String, String> result = this.getCacheKey(pjp, cache);
         String key = (String)result.get("key");
         var7 = this.getRealType(linkRedisClient.get(key), pjp.getMethod());
      } finally {
         RedisUtil.returnResource(linkRedisClient);
      }

      return var7;
   }

   private void asycSaveObjToCache(final String key, final CacheAopProxyChain pjp, final Object obj, final LinkCache cache) {
      LinkThreadPoolExecutor linkThreadPoolExecutor = (LinkThreadPoolExecutor)BeanTools.getBean(LinkThreadPoolExecutor.class);
      linkThreadPoolExecutor.execute(new Runnable() {
         public void run() {
            LinkRedisClient linkRedisClient = null;
            String tempKey = key;

            try {
               linkRedisClient = RedisUtil.getJedis();
               if (StringUtils.isBlank(tempKey) && pjp == null && cache == null) {
                  throw new Exception("生成缓存key出错");
               }

               if (StringUtils.isBlank(tempKey) && pjp != null && cache != null) {
                  Map<String, String> result = CacheHandler.this.getCacheKey(pjp, cache);
                  tempKey = (String)result.get("key");
               }

               Object tempObj = obj;
               if (tempObj == null && pjp != null) {
                  tempObj = CacheHandler.this.getData(pjp, pjp.getArgs());
               }

               if (tempObj == null) {
                  throw new Exception("获取缓存对象失败");
               }

               if (linkRedisClient.hexists("CACHE_KEY", cache.expire())) {
                  linkRedisClient.set(tempKey, JSON.toJSONString(tempObj));
                  linkRedisClient.expire(tempKey, Integer.valueOf(linkRedisClient.hget("CACHE_KEY", cache.expire())));
               }
            } catch (Exception var8) {
               Exception e = var8;
               CacheHandler.this.logger.error(e.getMessage(), e);
            } catch (Throwable var9) {
               Throwable throwable = var9;
               CacheHandler.this.logger.error(throwable.getMessage(), throwable);
            } finally {
               RedisUtil.returnResource(linkRedisClient);
            }

         }
      });
   }

   private Map<String, String> getCacheKey(CacheAopProxyChain pjp, LinkCache cache) throws Exception {
      Object[] args = pjp.getArgs();
      String userIdentity = "";
      List<Object> list = new ArrayList();
      String reloadCache = "N";
      list.add(pjp.getTarget().getClass().getName() + "_" + pjp.getMethod().getName());
      if (args != null && args.length > 0) {
         Object[] var7 = args;
         int var8 = args.length;

         for(int var9 = 0; var9 < var8; ++var9) {
            Object obj = var7[var9];
            if (!(obj instanceof HttpSession) && !(obj instanceof ServletRequest) && !(obj instanceof ServletResponse)) {
               list.add(obj);
            }

            if (obj instanceof BasicModel) {
               reloadCache = "Y".equals(reloadCache) ? reloadCache : ((BasicModel)obj).getReloadCache();
               ((BasicModel)obj).setReloadCache((String)null);
            }
         }
      }

      if (cache.type() == CacheType.PRIVATE) {
         CoreUser user = UserUtil.getUser();
         if (user == null) {
            userIdentity = "public";
         } else {
            userIdentity = user.getId() + "_" + user.getPostnId() + "_" + user.getOrgId();
         }
      } else {
         userIdentity = "public";
      }

      list.add(userIdentity);
      String json = JSON.toJSONString(list);
      this.logger.info("数据序列化");
      String key = PasswordUtil.getMD5StrJava(json);
      this.logger.info("加密后的字符串：" + key);
      Map<String, String> result = new HashMap(2);
      String expire = cache.expire();
      result.put("key", "lnk_dc_" + expire + "_" + cache.type() + "_" + key);
      result.put("reloadCache", reloadCache);
      return result;
   }

   public Object proceed(CacheAopProxyChain pjp, LinkCache cache) throws Throwable {
      Object[] arguments = pjp.getArgs();
      OpType opType = cache.opType();
      if (opType != OpType.NO_CACHE && this.isCacheable(cache, pjp)) {
         if (opType == OpType.NEEDS) {
            return this.getDataByNeeds(pjp, cache);
         } else if (opType == OpType.LAST_TIME) {
            return this.getDataByLast(pjp, cache);
         } else if (opType == OpType.HAS_READ) {
            return this.getDataByHasRead(pjp, cache);
         } else {
            throw new Exception("不存在缓存操作类型");
         }
      } else {
         return this.getData(pjp);
      }
   }

   private boolean isCacheable(LinkCache cache, CacheAopProxyChain pjp) throws Exception {
      if (cache != null && pjp != null) {
         boolean rv = true;
         Object[] arguments = pjp.getArgs();
         CoreUser user = UserUtil.getUser();
         if (null != arguments && arguments.length > 0 && null != cache.condition() && cache.condition().length() > 0) {
            Bindings bindings = new SimpleBindings();
            bindings.put("args", arguments);
            if (user != null) {
               bindings.put("user", user);
            }

            Compilable compEngine = (Compilable)javaScriptEngine;
            CompiledScript script = compEngine.compile(cache.condition());
            return (Boolean)script.eval(bindings);
         } else {
            return rv;
         }
      } else {
         throw new Exception("不存在标签以及方法");
      }
   }

   public static void main(String[] args) throws ScriptException, NoSuchMethodException {
   }

   static {
      ScriptEngineManager manager = new ScriptEngineManager();
      ScriptEngine scriptEngine = manager.getEngineByName("javascript");
      if (scriptEngine == null) {
         throw new RuntimeException("获取JavaScript解析引擎失败");
      } else {
         javaScriptEngine = scriptEngine;
      }
   }
}
